@@ -955,7 +955,7 @@ ev_application_startup (GApplication *gapplication)
           "win.delete-bookmark",        "<Ctrl><Shift>D", NULL,
           "win.close",                  "<Ctrl>W", NULL,
           "win.escape",                 "Escape", NULL,
-          "win.find",                   "<Ctrl>F", "slash", NULL,
+          "win.find",                   "<Ctrl>F", "slash", "KP_Divide", NULL,
           "win.find-next",              "<Ctrl>G", "F3", NULL,
           "win.find-previous",          "<Ctrl><Shift>G", "<Shift>F3", NULL,
           "win.select-page",            "<Ctrl>L", NULL,
@@ -968,7 +968,7 @@ ev_application_startup (GApplication *gapplication)
           "win.sizing-mode::fit-page",  "f", NULL,
           "win.sizing-mode::fit-width", "w", NULL,
           "win.sizing-mode::automatic", "a", NULL,
-          "win.default-zoom",           "<Ctrl>0", NULL,
+          "win.default-zoom",           "<Ctrl>0", "<Ctrl>KP_0", NULL,
           "win.toggle-menu",            "F10", NULL,
           "win.caret-navigation",       "F7", NULL,
           "win.zoom-in",                "plus", "<Ctrl>plus", "KP_Add", "<Ctrl>KP_Add", "equal", "<Ctrl>equal", NULL,
